@@ -92,8 +92,13 @@ license_dialog (const gchar *text)
                             (_("License"),
                             NULL,
                             GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+#ifdef G_OS_WIN32
                             GTK_STOCK_OK, GTK_RESPONSE_OK,
                             GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+#else /* not G_OS_WIN32 */
+                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                            GTK_STOCK_OK, GTK_RESPONSE_OK,
+#endif /* not G_OS_WIN32 */
                             NULL);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
   gtk_window_set_default_size (GTK_WINDOW (dialog),
