@@ -1,6 +1,6 @@
 /*
     misc
-    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include "misc.h"
+#include <gdk/gdkkeysyms.h>
 
 
 /******************************************************************************
@@ -33,8 +34,9 @@ misc_create_menu (MiscCreateMenuEntry *entries,
 {
   gint i;
 
-  for (i = 0; entries[i].menu_type != MISC_CREATE_MENU_TERMINAL; i++)
-    switch (entries[i].menu_type)
+  for (i = 0; (entries[i].menu_type & MISC_CREATE_MENU_MASK)
+                                            != MISC_CREATE_MENU_TERMINAL; i++)
+    switch (entries[i].menu_type & MISC_CREATE_MENU_MASK)
       {
         case MISC_CREATE_MENU_BAR:
           entries[i].widget = gtk_menu_bar_new ();
@@ -58,15 +60,76 @@ misc_create_menu (MiscCreateMenuEntry *entries,
             g_free (path);
           }
           break;
-        case MISC_CREATE_MENU_ITEM:
-        case MISC_CREATE_MENU_SEPARATOR:
+        default:
           {
             gchar *path, *p;
             gint j;
 
-            entries[i].widget = entries[i].name
-                        ? gtk_menu_item_new_with_mnemonic (_(entries[i].name))
-                        : gtk_separator_menu_item_new ();
+            switch (entries[i].menu_type & MISC_CREATE_MENU_MASK)
+              {
+                case MISC_CREATE_MENU_STOCK:
+                  if (entries[i].menu_type & MISC_CREATE_MENU_NOIMAGE)
+                    {
+                      GtkStockItem stock_item;
+
+                      gtk_stock_lookup (entries[i].name, &stock_item);
+                      entries[i].widget = gtk_image_menu_item_new_with_mnemonic
+                                                            (stock_item.label);
+                    }
+                  else
+                    {
+                      entries[i].widget = gtk_image_menu_item_new_from_stock
+                                        (entries[i].name,
+                                        entries[i].accel ? NULL : accel_group);
+                    }
+                  break;
+                case MISC_CREATE_MENU_ITEM:
+                  if (entries[i].xpm
+                        && !(entries[i].menu_type & MISC_CREATE_MENU_NOIMAGE))
+                    {
+                      GdkPixbuf *pixbuf;
+
+                      entries[i].widget = gtk_image_menu_item_new_with_mnemonic
+                                                        (_(entries[i].name));
+                      pixbuf = gdk_pixbuf_new_from_xpm_data (entries[i].xpm);
+                      gtk_image_menu_item_set_image
+                                    (GTK_IMAGE_MENU_ITEM (entries[i].widget),
+                                        gtk_image_new_from_pixbuf (pixbuf));
+                      g_object_unref (pixbuf);
+                    }
+                  else
+                    {
+                      entries[i].widget = gtk_menu_item_new_with_mnemonic
+                                                        (_(entries[i].name));
+                    }
+                  break;
+                case MISC_CREATE_MENU_CHECK:
+                  entries[i].widget = gtk_check_menu_item_new_with_mnemonic
+                                                        (_(entries[i].name));
+                  break;
+                case MISC_CREATE_MENU_RADIO:
+                  entries[i].widget = gtk_radio_menu_item_new_with_mnemonic
+                        (i > 0
+                        && (entries[i - 1].menu_type & MISC_CREATE_MENU_MASK)
+                                                     == MISC_CREATE_MENU_RADIO
+                            ? gtk_radio_menu_item_get_group
+                                (GTK_RADIO_MENU_ITEM (entries[i - 1].widget))
+                            : NULL, _(entries[i].name));
+                  break;
+                default:
+                  entries[i].widget = gtk_separator_menu_item_new ();
+              }
+            if (entries[i].menu_type & MISC_CREATE_MENU_ELLIPSIS)
+              {
+                gchar *mnemonic;
+                GtkWidget *label;
+
+                label = gtk_bin_get_child (GTK_BIN (entries[i].widget));
+                mnemonic = g_strconcat
+                        (gtk_label_get_label (GTK_LABEL (label)), "...", NULL);
+                gtk_label_set_text_with_mnemonic (GTK_LABEL (label), mnemonic);
+                g_free (mnemonic);
+              }
             path = g_strdup (entries[i].path);
             p = g_strrchr (path, '/') + 1 ;
             *p = '\0';
@@ -78,7 +141,7 @@ misc_create_menu (MiscCreateMenuEntry *entries,
                   break;
                 }
             g_free (path);
-            if (accel_group && entries[i].accel)
+            if (accel_group && entries[i].accel && entries[i].accel[0] != '\0')
               {
                 guint key;
                 GdkModifierType mods;
@@ -89,8 +152,15 @@ misc_create_menu (MiscCreateMenuEntry *entries,
                                         GTK_ACCEL_VISIBLE | GTK_ACCEL_LOCKED);
               }
             if (entries[i].func)
-              g_signal_connect (G_OBJECT (entries[i].widget), "activate",
+              g_signal_connect (G_OBJECT (entries[i].widget),
+                            (entries[i].menu_type & MISC_CREATE_MENU_MASK)
+                                                    == MISC_CREATE_MENU_CHECK
+                            || (entries[i].menu_type & MISC_CREATE_MENU_MASK)
+                                                    == MISC_CREATE_MENU_RADIO
+                                                    ? "toggled" : "activate",
                         G_CALLBACK (entries[i].func), entries[i].user_data);
+            if (entries[i].menu_type & MISC_CREATE_MENU_DISABLE)
+              gtk_widget_set_sensitive (entries[i].widget, FALSE);
           }
       }
 }
@@ -133,23 +203,69 @@ misc_create_toolbar (MiscCreateToolbarEntry *entries)
     {
       if (entries[i].name)
         {
-          GdkPixbuf *pixbuf;
+          if (entries[i].xpm)
+            {
+              GdkPixbuf *pixbuf;
 
-          pixbuf = gdk_pixbuf_new_from_xpm_data (entries[i].xpm);
-          entries[i].tool_item = gtk_tool_button_new
+              pixbuf = gdk_pixbuf_new_from_xpm_data (entries[i].xpm);
+#if GTK_CHECK_VERSION(2,4,0)
+              entries[i].tool_item = gtk_tool_button_new
                     (gtk_image_new_from_pixbuf (pixbuf), _(entries[i].name));
-          g_object_unref (pixbuf);
+              gtk_tool_item_set_tooltip (entries[i].tool_item, tooltips,
+                                    _(entries[i].name), _(entries[i].name));
+#else /* not GTK_CHECK_VERSION(2,4,0) */
+              entries[i].tool_item = gtk_toolbar_append_item
+                                            (GTK_TOOLBAR (toolbar),
+                                            _(entries[i].name),
+                                            _(entries[i].name),
+                                            _(entries[i].name),
+                                            gtk_image_new_from_pixbuf (pixbuf),
+                                            G_CALLBACK (entries[i].func),
+                                            NULL);
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
+              g_object_unref (pixbuf);
+            }
+          else
+            {
+              gchar *text;
+              GtkStockItem stock_item;
+
+              gtk_stock_lookup (entries[i].name, &stock_item);
+              text = misc_mnemonic_to_text (stock_item.label);
+#if GTK_CHECK_VERSION(2,4,0)
+              entries[i].tool_item = gtk_tool_button_new_from_stock
+                                                            (entries[i].name);
+              gtk_tool_item_set_tooltip (entries[i].tool_item, tooltips,
+                                                                text, text);
+#else /* not GTK_CHECK_VERSION(2,4,0) */
+              entries[i].tool_item = gtk_toolbar_insert_stock
+                                                (GTK_TOOLBAR (toolbar),
+                                                entries[i].name,
+                                                text,
+                                                text,
+                                                G_CALLBACK (entries[i].func),
+                                                NULL,
+                                                -1);
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
+              g_free (text);
+            }
+#if GTK_CHECK_VERSION(2,4,0)
           if (entries[i].func)
             g_signal_connect (G_OBJECT (entries[i].tool_item), "clicked",
                         G_CALLBACK (entries[i].func), entries[i].user_data);
-          gtk_tool_item_set_tooltip (entries[i].tool_item, tooltips,
-                                    _(entries[i].name), _(entries[i].name));
+#endif /* GTK_CHECK_VERSION(2,4,0) */
         }
       else
         {
+#if GTK_CHECK_VERSION(2,4,0)
           entries[i].tool_item = gtk_separator_tool_item_new ();
+#else /* not GTK_CHECK_VERSION(2,4,0) */
+          gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
         }
+#if GTK_CHECK_VERSION(2,4,0)
       gtk_toolbar_insert (GTK_TOOLBAR (toolbar), entries[i].tool_item, i);
+#endif /* GTK_CHECK_VERSION(2,4,0) */
     }
   return toolbar;
 }
@@ -177,6 +293,34 @@ misc_find_toolbar (MiscCreateToolbarEntry *entries,
 * ja:低レベル関数群                                                           *
 *                                                                             *
 ******************************************************************************/
+/*  ja:mnemonicをテキストに変換する
+    mnemonic,mnemonic
+         RET,テキスト                                                       */
+gchar *
+misc_mnemonic_to_text (const gchar *mnemonic)
+{
+  gchar *text;
+  gint i;
+
+  if (!mnemonic)
+    return NULL;
+  text = g_strdup (mnemonic);
+  for (i = 0; text[i] != '\0'; i++)
+    if (text[i] ==  '(' && text[i + 1] ==  '_'
+                    && g_ascii_isalpha (text[i + 2]) && text[i + 3] ==  ')')
+      {
+        g_memmove (text + i, text + i + 4, g_strlen (text + i) - 3);
+        break;
+      }
+    else if (text[i] == '_' && g_ascii_isalpha (text[i + 1]))
+      {
+        g_memmove (text + i, text + i + 2, g_strlen (text + i) - 1);
+        break;
+      }
+  return text;
+}
+
+
 /*  ja:スクロールバーを設定する
        scroll,スクロールウイジット
          func,シグナル関数
@@ -265,6 +409,18 @@ misc_scroll_window (GtkWidget  *widget,
 }
 
 
+/* ja:ESCが押されたとき */
+gboolean
+misc_dialog_key_press (GtkWidget   *widget,
+                       GdkEventKey *event,
+                       gpointer     user_data)
+{
+  if (event->keyval == GDK_Escape)
+    gtk_dialog_response (GTK_DIALOG (widget), GTK_RESPONSE_NONE);
+  return FALSE;
+}
+
+
 /******************************************************************************
 *                                                                             *
 * ja:数値文字列関数群                                                         *
