@@ -1,6 +1,6 @@
 /*
     orzconf
-    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 */
 #include "orzconf.h"
 #include "orzhistory.h"
+#include "misc/misc.h"
 #include <gdk/gdkkeysyms.h>
 
 
@@ -146,7 +147,9 @@ orz_conf_delete (GtkWidget *widget,
 static void
 orz_conf_init (OrzConf *conf)
 {
+  gchar *title;
   gint width, height;
+  GtkStockItem stock_item;
   GtkWidget *label0, *label1, *label2;
   GtkWidget *frame0, *frame1, *hbox0, *hbox1, *vbox;
 
@@ -154,14 +157,24 @@ orz_conf_init (OrzConf *conf)
   height = gdk_screen_height ();
 
   /* ja:メインウインドウ */
-  gtk_window_set_title (GTK_WINDOW (conf), _("Configuration"));
+  gtk_stock_lookup (GTK_STOCK_PREFERENCES, &stock_item);
+  title = misc_mnemonic_to_text (stock_item.label);
+  gtk_window_set_title (GTK_WINDOW (conf), title);
+  g_free (title);
   g_signal_connect (G_OBJECT (conf), "delete-event",
                                         G_CALLBACK (orz_conf_delete), NULL);
   /* ja:ボタン */
+#ifdef G_OS_WIN32
   conf->ok_button = gtk_dialog_add_button (GTK_DIALOG (conf),
                                         GTK_STOCK_OK, GTK_RESPONSE_OK);
   conf->cancel_button = gtk_dialog_add_button (GTK_DIALOG (conf),
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+#else /* not G_OS_WIN32 */
+  conf->cancel_button = gtk_dialog_add_button (GTK_DIALOG (conf),
+                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+  conf->ok_button = gtk_dialog_add_button (GTK_DIALOG (conf),
+                                        GTK_STOCK_OK, GTK_RESPONSE_OK);
+#endif /* not G_OS_WIN32 */
   /* ja:チェックボタン */
   conf->newfile_check = gtk_check_button_new_with_mnemonic (_("_New File"));
   conf->second_check  = gtk_check_button_new_with_mnemonic (_("_Open Window"));
