@@ -1,6 +1,6 @@
 /*
     orzmdi
-    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include "misc/fileio.h"
 #include "orzmdi.h"
@@ -33,11 +33,18 @@ enum
   CHANGE_WINDOW_SIGNAL,
   CLOSE_WINDOW_SIGNAL,
   REMOVE_WINDOW_SIGNAL,
+  REORDER_WINDOW_SIGNAL,
   SWITCH_WINDOW_SIGNAL,
   LAST_SIGNAL
 };
 
 
+enum
+{
+  TARGET_ORZ_MDI_TAB
+};
+
+
 struct _OrzMdiTab
 {
   gpointer page_data;
@@ -45,7 +52,7 @@ struct _OrzMdiTab
   gboolean edited;      /* ja:TRUE:編集済み,FALSE:未編集 */
   gchar *file;
   gint same;            /* -2:名前のみ,-1:フルパス,0...:フルパス+数字 */
-  GtkWidget *image, *label;
+  GtkWidget *child, *image, *label;
   OrzMdi *mdi;
 };
 
@@ -63,13 +70,21 @@ orz_mdi_label_markup (OrzMdiTab *tab)
 
   if (tab->same < -1)
     {
+#if GLIB_CHECK_VERSION(2,6,0)
+      label = g_filename_display_basename (tab->file);
+#else /* not GLIB_CHECK_VERSION(2,6,0) */
       str = g_path_get_basename (tab->file);
       label = g_filename_to_utf8 (str, -1, NULL, NULL, NULL);
       g_free (str);
+#endif /* not GLIB_CHECK_VERSION(2,6,0) */
     }
   else
     {
+#if GLIB_CHECK_VERSION(2,6,0)
+      label = g_filename_display_name (tab->file);
+#else /* not GLIB_CHECK_VERSION(2,6,0) */
       label = g_filename_to_utf8 (tab->file, -1, NULL, NULL, NULL);
+#endif /* not GLIB_CHECK_VERSION(2,6,0) */
     }
   if (tab->edited)
     {
@@ -236,7 +251,7 @@ orz_mdi_get_type (void)
 
   if (!orz_mdi_type)
     {
-      static const GTypeInfo mdi_info =
+      const static GTypeInfo mdi_info =
       {
         sizeof (OrzMdiClass),
         NULL,               /* base_init */
@@ -344,6 +359,16 @@ orz_mdi_class_init (OrzMdiClass *klass)
                                 G_TYPE_NONE, 2,
                                 G_TYPE_UINT,
                                 G_TYPE_POINTER);
+  orz_mdi_signals[REORDER_WINDOW_SIGNAL]
+                = g_signal_new ("reorder_window",
+                                G_TYPE_FROM_CLASS (gobject_class),
+                                G_SIGNAL_RUN_LAST,
+                                G_STRUCT_OFFSET (OrzMdiClass, reorder_window),
+                                NULL, NULL,
+                                g_cclosure_marshal_VOID__UINT_POINTER,
+                                G_TYPE_NONE, 2,
+                                G_TYPE_UINT,
+                                G_TYPE_POINTER);
   orz_mdi_signals[SWITCH_WINDOW_SIGNAL]
                 = g_signal_new ("switch_window",
                                 G_TYPE_FROM_CLASS (gobject_class),
@@ -387,29 +412,11 @@ orz_mdi_clicked (GtkWidget *widget,
 
   tab = g_object_get_data (G_OBJECT (tab_label), "user_data");
   if (tab)
-    {
-      gint i;
-      GList *glist;
-
-      glist = gtk_container_children (GTK_CONTAINER (tab->mdi));
-      for (i = g_list_length (glist) - 1; i >= 0; i--)
-        {
-          GtkWidget *child;
-
-          child = g_list_nth_data (glist, i);
-          if (gtk_notebook_get_tab_label (GTK_NOTEBOOK (tab->mdi), child)
-                                                                == tab_label)
-            {
-              g_signal_emit (tab->mdi,
-                     orz_mdi_signals[CLOSE_WINDOW_SIGNAL],
-                     0,
-                     gtk_notebook_page_num (GTK_NOTEBOOK (tab->mdi), child),
-                     tab->page_data);
-              break;
-            }
-        }
-      g_list_free (glist);
-    }
+    g_signal_emit (tab->mdi,
+           orz_mdi_signals[CLOSE_WINDOW_SIGNAL],
+           0,
+           gtk_notebook_page_num (GTK_NOTEBOOK (tab->mdi), tab->child),
+           tab->page_data);
 }
 
 
@@ -442,6 +449,67 @@ orz_mdi_destroy (GtkWidget *widget,
 }
 
 
+/* ja:ドロップされたとき */
+void
+orz_mdi_tab_drag_begin (GtkWidget      *widget,
+                        GdkDragContext *context,
+                        gpointer        user_data)
+{
+  GdkPixbuf *pixbuf;
+  OrzMdiTab *tab;
+
+  tab = g_object_get_data (G_OBJECT (widget), "user_data");
+  pixbuf = fileio_extract_icon (tab->file, FILEIO_ICON_SIZE_LARGE);
+  if (pixbuf)
+    {
+      gtk_drag_source_set_icon_pixbuf (widget, pixbuf);
+      g_object_unref (pixbuf);
+    }
+}
+static void
+orz_mdi_tab_drag_data_get (GtkWidget        *widget,
+                           GdkDragContext   *context,
+                           GtkSelectionData *data,
+                           guint             info,
+                           guint             time,
+                           gpointer          user_data)
+{
+  const static gchar *text = "dummy";
+
+  if (info == TARGET_ORZ_MDI_TAB)
+    gtk_selection_data_set (data, data->target, 8,
+                                        (const guchar *)text, g_strlen (text));
+}
+static void
+orz_mdi_tab_drag_data_received (GtkWidget        *widget,
+                                GdkDragContext   *context,
+                                gint              x,
+                                gint              y,
+                                GtkSelectionData *data,
+                                guint             info,
+                                guint             time,
+                                gpointer          user_data)
+{
+  if (info == TARGET_ORZ_MDI_TAB)
+    {
+      gint page_num;
+      OrzMdiTab *tab;
+
+      tab = g_object_get_data (G_OBJECT (widget), "user_data");
+      page_num = gtk_notebook_page_num (GTK_NOTEBOOK (tab->mdi), tab->child);
+      tab = g_object_get_data
+                (G_OBJECT (gtk_drag_get_source_widget (context)), "user_data");
+      gtk_notebook_reorder_child (GTK_NOTEBOOK (tab->mdi), tab->child,
+                                                                    page_num);
+      g_signal_emit (tab->mdi,
+                     orz_mdi_signals[REORDER_WINDOW_SIGNAL],
+                     0,
+                     page_num,
+                     tab->page_data);
+    }
+}
+
+
 /*  ja:ページのタブを作る
           mdi,ウィジェット
         child,内容
@@ -450,26 +518,18 @@ orz_mdi_destroy (GtkWidget *widget,
     page_data,ページデータ
           RET,ウィジェット                                                  */
 static GtkWidget *
-orz_mdi_create_tab_label (OrzMdi      *mdi,
-                          GtkWidget   *child,
-                          const gchar *file,
-                          gboolean     created,
-                          gpointer     page_data)
+orz_mdi_create_tab_label (OrzMdi         *mdi,
+                          GtkWidget      *child,
+                          const gchar    *file,
+                          const gboolean  created,
+                          gpointer        page_data)
 {
   GdkPixbuf *pixbuf;
-  GtkWidget *button, *frame, *hbox, *event;
+  GtkIconSet *icon_set;
+  GtkWidget *image = NULL, *button, *hbox, *event;
+  GtkTargetEntry dnd_entries[1] = {{"ORZ_MDI_TAB",
+                                    GTK_TARGET_SAME_APP, TARGET_ORZ_MDI_TAB}};
   OrzMdiTab *tab;
-  static const gchar *xpm[] = {
-"8 7 2 1",
-"   c None",
-".  c #000000",
-"..    ..",
-" ..  .. ",
-"  ....  ",
-"   ..   ",
-"  ....  ",
-" ..  .. ",
-"..    .."};
 
   if (!mdi)
     return NULL;
@@ -480,23 +540,116 @@ orz_mdi_create_tab_label (OrzMdi      *mdi,
                                                             : g_strdup ("");
   tab->same = -2;
   tab->mdi = mdi;
+  tab->child = child;
   file_add_edit (mdi, tab);
+  /* ja:イメージ */
+  icon_set = gtk_icon_factory_lookup_default (GTK_STOCK_CLOSE);
+  if (icon_set)
+    {
+      pixbuf = gtk_icon_set_render_icon (icon_set,
+                                         gtk_widget_get_default_style (),
+                                         GTK_TEXT_DIR_NONE,
+                                         GTK_STATE_NORMAL,
+                                         GTK_ICON_SIZE_BUTTON,
+                                         NULL, NULL);
+      if (pixbuf)
+        {
+          if (gdk_pixbuf_get_colorspace (pixbuf) == GDK_COLORSPACE_RGB
+                            && gdk_pixbuf_get_n_channels (pixbuf) == 4
+                            && gdk_pixbuf_get_has_alpha (pixbuf)
+                            && gdk_pixbuf_get_bits_per_sample (pixbuf) == 8)
+            {
+              int width, height, rowstride;
+              gint x, y, left, top, right, bottom;
+              guchar *pixels;
+
+              pixels = gdk_pixbuf_get_pixels (pixbuf);
+              width = gdk_pixbuf_get_width (pixbuf);
+              height = gdk_pixbuf_get_height (pixbuf);
+              rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+              for (x = 0; x < width; x++)
+                for (y = 0; y < height; y++)
+                  if (pixels[y * rowstride + x * 4 + 3] > 0)
+                    goto loop_left;
+              loop_left: left = x;
+              for (y = 0; y < height; y++)
+                for (x = 0; x < width; x++)
+                  if (pixels[y * rowstride + x * 4 + 3] > 0)
+                    goto loop_top;
+              loop_top: top = y;
+              for (x = width - 1; x >= 0; x--)
+                for (y = 0; y < height; y++)
+                  if (pixels[y * rowstride + x * 4 + 3] > 0)
+                    goto loop_right;
+              loop_right: right = x;
+              for (y = height - 1; y >= 0; y--)
+                for (x = 0; x < width; x++)
+                  if (pixels[y * rowstride + x * 4 + 3] > 0)
+                    goto loop_bottom;
+              loop_bottom: bottom = y;
+              if (left <= right && top <= bottom)
+                {
+                  int w, h, r;
+                  guchar *p;
+                  gint i;
+                  GdkPixbuf *b;
+
+                  w = right - left + 1;
+                  h = bottom - top + 1;
+                  b = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, w, h);
+                  p = gdk_pixbuf_get_pixels (b);
+                  r = gdk_pixbuf_get_rowstride (b);
+                  for (x = 0; x < w; x++)
+                    for (y = 0; y < h; y++)
+                      for (i = 0; i < 4; i++)
+                        p[y * r + x * 4 + i]
+                        = pixels[(y + top) * rowstride + (x + left) * 4 + i];
+                  image = gtk_image_new_from_pixbuf (b);
+                  g_object_unref (b);
+                }
+            }
+          g_object_unref (pixbuf);
+        }
+    }
+  if (!image)
+    image = gtk_image_new_from_stock (GTK_STOCK_CDROM, GTK_ICON_SIZE_BUTTON);
+  if (!image)
+    {
+      const static gchar *xpm[] = {
+"12 13 4 1",
+"   c None",
+"X  c #000000",
+"+  c #808080",
+".  c #FFFFFF",
+"+++++++++++.",
+"+.........+.",
+"+.        +.",
+"+.XX    XX+.",
+"+. XX  XX +.",
+"+.  XXXX  +.",
+"+.   XX   +.",
+"+.  XXXX  +.",
+"+. XX  XX +.",
+"+.XX    XX+.",
+"+.        +.",
+"+++++++++++.",
+"............"};
+
+      pixbuf = gdk_pixbuf_new_from_xpm_data (xpm);
+      image = gtk_image_new_from_pixbuf (pixbuf);
+      g_object_unref (pixbuf);
+    }
   /* ja:ボタン */
   button = gtk_button_new ();
-  frame = gtk_frame_new (NULL);
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-  pixbuf = gdk_pixbuf_new_from_xpm_data (xpm);
-  gtk_container_add (GTK_CONTAINER (frame),
-                                        gtk_image_new_from_pixbuf (pixbuf));
-  g_object_unref (pixbuf);
-  gtk_container_add (GTK_CONTAINER (button), frame);
+  gtk_container_add (GTK_CONTAINER (button), image);
   gtk_widget_show_all (button);
   /* ja:ラベル */
   tab->label = gtk_label_new ("");
   orz_mdi_label_markup (tab);
   gtk_widget_show (tab->label);
   /* ja:アイコン */
-  pixbuf = fileio_extract_icon (tab->file);
+  pixbuf = fileio_extract_icon (tab->file, FILEIO_ICON_SIZE_SMALL);
   if (pixbuf)
     {
       tab->image = gtk_image_new_from_pixbuf (pixbuf);
@@ -520,6 +673,18 @@ orz_mdi_create_tab_label (OrzMdi      *mdi,
                                         G_CALLBACK (orz_mdi_clicked), event);
   g_signal_connect (G_OBJECT (child), "destroy",
                                         G_CALLBACK (orz_mdi_destroy), mdi);
+  /* en:Drag & Drop */
+  gtk_drag_source_set (event, GDK_BUTTON1_MASK, dnd_entries, 1,
+                                                            GDK_ACTION_MOVE);
+  gtk_drag_dest_set (event, GTK_DEST_DEFAULT_ALL, dnd_entries, 1,
+                                                            GDK_ACTION_MOVE);
+  g_signal_connect (G_OBJECT (event), "drag_begin",
+                            G_CALLBACK (orz_mdi_tab_drag_begin), NULL);
+  g_signal_connect (G_OBJECT (event), "drag_data_get",
+                            G_CALLBACK (orz_mdi_tab_drag_data_get), NULL);
+  g_signal_connect (G_OBJECT (event), "drag_data_received",
+                            G_CALLBACK (orz_mdi_tab_drag_data_received), NULL);
+
   return event;
 }
 
@@ -534,13 +699,13 @@ orz_mdi_create_tab_label (OrzMdi      *mdi,
     page_data,ページデータ
        RET,ページ(0...,-1:エラー)                                           */
 gint
-orz_mdi_insert_page_menu (OrzMdi      *mdi,
-                          GtkWidget   *child,
-                          GtkWidget   *menu,
-                          gint         position,
-                          const gchar *file,
-                          gboolean     created,
-                          gpointer     page_data)
+orz_mdi_insert_page_menu (OrzMdi         *mdi,
+                          GtkWidget      *child,
+                          GtkWidget      *menu,
+                          const gint      position,
+                          const gchar    *file,
+                          const gboolean  created,
+                          gpointer        page_data)
 {
   gint page_num = -1;
   GtkWidget *tab_label;
@@ -610,8 +775,8 @@ orz_mdi_get_page_from_data (OrzMdi  *mdi,
     page_num,ページ(0...),-1:カレント
          RET,ページ内部データ                                               */
 static OrzMdiTab *
-orz_mdi_get_tab (OrzMdi *mdi,
-                 gint    page_num)
+orz_mdi_get_tab (OrzMdi     *mdi,
+                 const gint  page_num)
 {
   if (mdi)
     {
@@ -637,13 +802,13 @@ orz_mdi_get_tab (OrzMdi *mdi,
     page_num,ページ(0...),-1:カレント
          RET,ページデータ                                                   */
 gpointer
-orz_mdi_get_data (OrzMdi *mdi,
-                  gint    page_num)
+orz_mdi_get_data (OrzMdi     *mdi,
+                  const gint  page_num)
 {
   OrzMdiTab *tab;
 
   tab = orz_mdi_get_tab (mdi, page_num);
-  return tab ? tab->page_data: NULL;
+  return tab ? tab->page_data : NULL;
 }
 
 
@@ -653,9 +818,9 @@ orz_mdi_get_data (OrzMdi *mdi,
      page_data,ページデータ
            RET,TRUE:正常終了,FALSE:エラー                                   */
 gboolean
-orz_mdi_set_data (OrzMdi  *mdi,
-                  gint     page_num,
-                  gpointer page_data)
+orz_mdi_set_data (OrzMdi     *mdi,
+                  const gint  page_num,
+                  gpointer    page_data)
 {
   OrzMdiTab *tab;
 
@@ -674,8 +839,8 @@ orz_mdi_set_data (OrzMdi  *mdi,
     page_num,ページ(0...),-1:カレント
          RET,ファイル名,NULL:エラー                                         */
 const gchar *
-orz_mdi_get_file (OrzMdi *mdi,
-                  gint    page_num)
+orz_mdi_get_file (OrzMdi     *mdi,
+                  const gint  page_num)
 {
   OrzMdiTab *tab;
 
@@ -691,7 +856,7 @@ orz_mdi_get_file (OrzMdi *mdi,
          RET,TRUE:正常終了,FALSE:エラー                                     */
 gboolean
 orz_mdi_set_file (OrzMdi      *mdi,
-                  gint         page_num,
+                  const gint   page_num,
                   const gchar *file)
 {
   if (mdi && file)
@@ -731,7 +896,8 @@ orz_mdi_set_file (OrzMdi      *mdi,
                   if (tab->image)
                     gtk_container_remove (GTK_CONTAINER (hbox), tab->image);
                   tab->image = NULL;
-                  pixbuf = fileio_extract_icon (tab->file);
+                  pixbuf = fileio_extract_icon (tab->file,
+                                                    FILEIO_ICON_SIZE_SMALL);
                   if (pixbuf)
                     {
                       tab->image = gtk_image_new_from_pixbuf (pixbuf);
@@ -760,8 +926,8 @@ orz_mdi_set_file (OrzMdi      *mdi,
     page_num,ページ(0...),-1:カレント
          RET,TRUE:新規作成,FALSE:既存ファイル                               */
 gboolean
-orz_mdi_get_created (OrzMdi *mdi,
-                     gint    page_num)
+orz_mdi_get_created (OrzMdi     *mdi,
+                     const gint  page_num)
 {
   OrzMdiTab *tab;
 
@@ -776,9 +942,9 @@ orz_mdi_get_created (OrzMdi *mdi,
     is_create,TRUE:新規作成,FALSE:既存ファイル
           RET,TRUE:正常終了,FALSE:エラー                                    */
 gboolean
-orz_mdi_set_created (OrzMdi   *mdi,
-                     gint      page_num,
-                     gboolean  is_create)
+orz_mdi_set_created (OrzMdi         *mdi,
+                     const gint      page_num,
+                     const gboolean  is_create)
 {
   OrzMdiTab *tab;
 
@@ -798,8 +964,8 @@ orz_mdi_set_created (OrzMdi   *mdi,
     page_num,ページ(0...),-1:カレント
          RET,TRUE:編集済み,FALSE:未編集                                     */
 gboolean
-orz_mdi_get_edited (OrzMdi *mdi,
-                    gint    page_num)
+orz_mdi_get_edited (OrzMdi     *mdi,
+                    const gint  page_num)
 {
   OrzMdiTab *tab;
 
@@ -814,9 +980,9 @@ orz_mdi_get_edited (OrzMdi *mdi,
      is_edit,TRUE:編集済み,FALSE:未編集
          RET,TRUE:正常終了,FALSE:エラー                                     */
 gboolean
-orz_mdi_set_edited (OrzMdi   *mdi,
-                    gint      page_num,
-                    gboolean  is_edit)
+orz_mdi_set_edited (OrzMdi         *mdi,
+                    const gint      page_num,
+                    const gboolean  is_edit)
 {
   OrzMdiTab *tab;
 
@@ -832,3 +998,44 @@ orz_mdi_set_edited (OrzMdi   *mdi,
     }
   return FALSE;
 }
+
+
+/*  ja:ページを移動する
+             mdi,ウィジェット
+    page_num_src,移動元ページ(0...),-1:カレント
+    page_num_dst,移動先ページ(0...),-1:末尾
+             RET,TRUE:正常終了,FALSE:エラー                                 */
+gboolean
+orz_mdi_reorder (OrzMdi     *mdi,
+                 const gint  page_num_src,
+                 const gint  page_num_dst)
+{
+  OrzMdiTab *tab;
+
+  tab = orz_mdi_get_tab (mdi, page_num_src);
+  if (!tab)
+    return FALSE;
+  gtk_notebook_reorder_child (GTK_NOTEBOOK (tab->mdi), tab->child,
+                                                                page_num_dst);
+  g_signal_emit (tab->mdi,
+                 orz_mdi_signals[REORDER_WINDOW_SIGNAL],
+                 0,
+                 page_num_dst,
+                 tab->page_data);
+  return TRUE;
+}
+
+
+/*  ja:ページの表示名を取得する
+         mdi,ウィジェット
+    page_num,ページ(0...),-1:カレント
+         RET,表示名                                                         */
+const gchar *
+orz_mdi_get_display_name (OrzMdi     *mdi,
+                          const gint  page_num)
+{
+  OrzMdiTab *tab;
+
+  tab = orz_mdi_get_tab (mdi, page_num);
+  return tab ? gtk_label_get_text (GTK_LABEL (tab->label)) : NULL;
+}
