@@ -42,12 +42,13 @@ void
 file_open_text (const gchar *file,
                 TmaidWindow *tmaid)
 {
-  gchar buf[16384], **charset, *ptr, *inbuf = NULL, *outbuf = NULL;
-  gssize bytes_read, bytes_written, leng, length = 0;
-  gint i, j, crlf[3];
+  gchar buf[16384], **charset, *inbuf = NULL, *outbuf = NULL;
+  gsize bytes_read, bytes_written;
+  gssize leng, length = 0;
+  gint i, j;
   gunichar c;
   FileIO *fio;
-  LineBuffer *p, *q;
+  LineBuffer *p;
   GError *error = NULL;
   GtkWidget *dialog;
 
@@ -153,6 +154,8 @@ file_open_text (const gchar *file,
   /* ja:変換できなければUTF-8とする */
   if (!outbuf)
     {
+      gchar *ptr;
+
       outbuf = inbuf;
       leng = length;
       while (userbreak && leng > 0
@@ -176,6 +179,8 @@ file_open_text (const gchar *file,
   /* ja:改行コード判定 */
   if (tmaid->ft.crlf >= 3)
     {
+      gint crlf[3];
+
       crlf[0] = crlf[1] = crlf[2] = 0;
       i = 0;
       while (userbreak && i < length)
@@ -236,6 +241,8 @@ file_open_text (const gchar *file,
       g_memmove (p->text, outbuf + i, p->length);
       if (j < length)
         {
+          LineBuffer *q;
+
           j += tmaid->ft.crlf == 0 ? 2 : 1;
           q = g_malloc0 (sizeof (LineBuffer));
           q->prev = p;
@@ -254,7 +261,7 @@ file_open_text (const gchar *file,
         while (gtk_events_pending ())
           gtk_main_iteration ();
         i = 0;
-        while (i <= p->length - 4)
+        while (i + 4 <= p->length)
           if (p->text[i++] == '&' && p->text[i++] == '#')
             {
               c = 0;
@@ -295,7 +302,7 @@ file_open_text (const gchar *file,
         while (gtk_events_pending ())
           gtk_main_iteration ();
         i = 0;
-        while (i <= p->length - 6)
+        while (i + 6 <= p->length)
           if (p->text[i++] == '\\' && p->text[i++] == 'u')
             {
               c = 0;
@@ -504,7 +511,7 @@ file_open_edit (const gchar *file)
   rc.height = tmaid->font_height;
   gtk_im_context_set_cursor_location (tmaid->im_context, &rc);
   /* ja:新規作成ではなく、ファイルが存在する時には履歴に加える */
-  if (file && fileio_isfile (file))
+  if (file && g_file_test (file, G_FILE_TEST_EXISTS))
     orz_history_add_file (ORZ_HISTORY (history), file);
   g_free (filename);
   /* ja:右マージン */
@@ -528,9 +535,8 @@ gboolean
 file_save_text (const gchar *file,
                 TmaidWindow *tmaid)
 {
-  gchar *buf = NULL, crlf[2], *ptr, *outbuf;
-  gssize bytes_read_prev, bytes_read, bytes_written;
-  gssize crlf_leng, leng, length = 0;
+  gchar *buf = NULL, crlf[2], *ptr;
+  gsize crlf_leng, length = 0;
   FileIO *fio;
   LineBuffer *p;
   GError *error = NULL;
@@ -538,7 +544,7 @@ file_save_text (const gchar *file,
 
   p = tmaid->start;
   while (p->prev)
-    p=p->prev;
+    p = p->prev;
   switch (tmaid->ft.crlf)
     {
       case  0: crlf[0] = '\r'; crlf[1] = '\n'; crlf_leng = 2; break;
@@ -552,13 +558,16 @@ file_save_text (const gchar *file,
   /* ja:書く */
   while (userbreak && p)
     {
+      gsize leng;
+
       if (!p->margin && p->next)
         {
           leng = p->length + crlf_leng;
           p->text = realloc (p->text, leng);
           g_memmove (p->text + p->length, crlf, crlf_leng);
         }
-      else if (!p->next && tmaid->ft.eof && p->text[p->length - 1] != '\x1a')
+      else if (!p->next && tmaid->ft.eof && p->length > 0
+                                        && p->text[p->length - 1] != '\x1a')
         {
           leng = p->length + 1;
           p->text = realloc (p->text, leng);
@@ -571,6 +580,10 @@ file_save_text (const gchar *file,
       ptr = p->text;
       while (userbreak && leng > 0)
         {
+          gchar *outbuf;
+          gsize bytes_read, bytes_written;
+          gssize bytes_read_prev;
+
           while (gtk_events_pending ())
             gtk_main_iteration ();
           bytes_read_prev = leng;
@@ -680,6 +693,8 @@ file_save_text (const gchar *file,
       ptr = buf;
       while (userbreak && length > 0)
         {
+          gssize leng;
+
           while (gtk_events_pending ())
             gtk_main_iteration ();
           leng = MIN (length, 16384);
@@ -688,6 +703,7 @@ file_save_text (const gchar *file,
               gtk_grab_remove (dialog);
               gtk_widget_destroy (dialog);
               fileio_close (fio);
+              g_free (buf);
               dialog = gtk_message_dialog_new (GTK_WINDOW (window),
                             GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                             GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
