@@ -106,10 +106,7 @@ ftype_dialog_changed (GtkTreeSelection *select,
   ftypedlg->row = -1;
   for (i = 0; gtk_tree_model_iter_nth_child (model, &iter, NULL, i); i++)
     if (gtk_tree_selection_iter_is_selected (select, &iter))
-      {
-        ftypedlg->row = i;
-        break;
-      }
+      ftypedlg->row = i;
   if (ftypedlg->row >= 0)
     {
       gtk_widget_set_sensitive (ftypedlg->button_add, TRUE);
@@ -417,8 +414,13 @@ ftype_dialog (GtkWindow *parent,
                             (ind ? _("Associating File Type") : _("File Type"),
                             parent,
                             GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+#ifdef G_OS_WIN32
+                            GTK_STOCK_OK, GTK_RESPONSE_OK,
+                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+#else /* not G_OS_WIN32 */
                             GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                             GTK_STOCK_OK, GTK_RESPONSE_OK,
+#endif /* not G_OS_WIN32 */
                             NULL);
   gtk_window_set_default_size (GTK_WINDOW (ftypedlg.dialog), 320, 240);
   g_signal_connect (G_OBJECT (ftypedlg.dialog), "key-press-event",
@@ -521,6 +523,7 @@ ftype_dialog (GtkWindow *parent,
 
   if (gtk_dialog_run (GTK_DIALOG (ftypedlg.dialog)) == GTK_RESPONSE_OK)
     {
+      /* ja:編集されているとき */
       GtkTreeIter iter;
       FileType *ft;
 
@@ -563,6 +566,12 @@ ftype_dialog (GtkWindow *parent,
               charlist_renewal_all (ft->charset, -1);
             }
         }
+      /* ja:フィルタのインデックス */
+      for (i = ftnum - 1; i >= 0; i--)
+        if (ftype[i].ft_id == findex)
+          break;
+      if (i < 0)
+        findex = -1;
       if (ftypedlg.check
         && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ftypedlg.check)))
         ftypedlg.row = ftnum;
