@@ -682,12 +682,17 @@ print_dialog_clicked_file (GtkWidget   *widget,
 # if GTK_CHECK_VERSION(2,4,0)
   dialog = gtk_file_chooser_dialog_new (title,
                     GTK_WINDOW (prndlg->dialog), GTK_FILE_CHOOSER_ACTION_SAVE,
+#ifdef G_OS_WIN32
+                    GTK_STOCK_SAVE_AS, GTK_RESPONSE_ACCEPT,
+                    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+#else /* not G_OS_WIN32 */
                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                     GTK_STOCK_SAVE_AS, GTK_RESPONSE_ACCEPT,
+#endif /* not G_OS_WIN32 */
                     NULL);
   file = g_filename_from_utf8 (gtk_entry_get_text
                     (GTK_ENTRY (prndlg->entry_file)), -1, NULL, NULL, NULL);
-  if (fileio_isfile (file))
+  if (g_file_test (file, G_FILE_TEST_EXISTS))
     {
       gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (dialog), file);
     }
@@ -698,7 +703,7 @@ print_dialog_clicked_file (GtkWidget   *widget,
       dir = g_path_get_dirname (file);
       path = fileio_get_full_path (dir);
       g_free (dir);
-      if (fileio_isfile (path))
+      if (g_file_test (path, G_FILE_TEST_EXISTS))
         gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), path);
       g_free (path);
     }
@@ -846,8 +851,13 @@ print_dialog (PrintConfig *prncfg)
                             (title,
                             GTK_WINDOW (window),
                             GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+#ifdef G_OS_WIN32
+                            GTK_STOCK_PRINT, GTK_RESPONSE_ACCEPT,
+                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+#else /* not G_OS_WIN32 */
                             GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                             GTK_STOCK_PRINT, GTK_RESPONSE_ACCEPT,
+#endif /* not G_OS_WIN32 */
                             NULL);
   g_free (title);
   g_signal_connect (G_OBJECT (prndlg.dialog), "key-press-event",
