@@ -553,8 +553,13 @@ property_dialog (GtkWindow *parent,
                             (title ? title : _("Details of File Type"),
                             parent,
                             GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+#ifdef G_OS_WIN32
+                            GTK_STOCK_OK, GTK_RESPONSE_OK,
+                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+#else /* not G_OS_WIN32 */
                             GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                             GTK_STOCK_OK, GTK_RESPONSE_OK,
+#endif /* not G_OS_WIN32 */
                             NULL);
   g_free (title);
   g_signal_connect (G_OBJECT (propdlg.dialog), "key-press-event",
