@@ -413,7 +413,8 @@ signal_expose (GtkWidget      *widget,
     {
       /* ja:選択範囲がないとき */
       start.x = -1;
-    }
+      start.y = end.x = end.y = 0;
+    } 
   else if (tmaid->select.y < tmaid->cursor.y
                                         || (tmaid->select.y == tmaid->cursor.y
                                         && tmaid->select.x < tmaid->cursor.x))
