@@ -229,7 +229,7 @@ main (int   argc,
   locale = g_build_filename (dirname, "locale", NULL);
   g_free (dirname);
 # ifdef G_OS_WIN32
-  if (!fileio_isfile (locale))
+  if (!g_file_test (locale, G_FILE_TEST_EXISTS))
     {
       g_free (locale);
       locale = g_win32_get_package_installation_subdirectory
@@ -690,9 +690,9 @@ TMAID_COPYRIGHT_NOTICE"\n"
                                         misc_create_toolbar (toolbar_entries));
   gtk_box_pack_start (GTK_BOX (vbox), handle, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), mdi, TRUE, TRUE, 0);
-  gtk_table_attach (GTK_TABLE (table), status, 0, 4, 0, 1,
+  gtk_table_attach (GTK_TABLE (table), frame, 0, 1, 0, 1,
             GTK_EXPAND | GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
-  gtk_table_attach (GTK_TABLE (table), frame, 4, 5, 0, 1,
+  gtk_table_attach (GTK_TABLE (table), status, 1, 5, 0, 1,
             GTK_EXPAND | GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
   gtk_box_pack_end (GTK_BOX (vbox), table, FALSE, FALSE, 0);
   gtk_box_pack_end (GTK_BOX (vbox), label, FALSE, FALSE, 0);
@@ -767,7 +767,10 @@ TMAID_COPYRIGHT_NOTICE"\n"
   g_key_file_set_integer (key_file, "tmaid", "n_pos", n_pos);
   g_key_file_set_integer (key_file, "tmaid", "ftnum", ftnum);
 #if GTK_CHECK_VERSION(2,4,0) || defined (G_OS_WIN32)
-  g_key_file_set_integer (key_file, "tmaid", "findex", findex);
+  for (i = ftnum - 1; i >= 0; i--)
+    if (ftype[i].ft_id == findex)
+      break;
+  g_key_file_set_integer (key_file, "tmaid", "findex", i);
 #endif /* GTK_CHECK_VERSION(2,4,0) || defined (G_OS_WIN32) */
   if (open_path)
     {
