@@ -68,7 +68,7 @@ typedef struct _TmaidHistory
 {
   gboolean caret;
   gchar *text;
-  gint length;              /* ja:文字数 */
+  gsize length;             /* ja:文字数 */
   GdkPoint cursor, select;
   struct _TmaidHistory *next;
 } TmaidHistory;
@@ -76,7 +76,7 @@ typedef struct _LineBuffer
 {
   gboolean margin;          /* ja:TRUE:右マージン,FALSE:通常の改行 */
   gchar *text;
-  gint length;              /* ja:文字数 */
+  gsize length;             /* ja:文字数 */
   struct _LineBuffer *prev, *next;
 } LineBuffer;
 typedef struct _TmaidWindow
